@@ -11,10 +11,11 @@ import (
 	"hash/fnv"
 	"io"
 	"io/fs"
+	"iter"
 	"os"
 	"path/filepath"
+	"slices"
 
-	"github.com/sinclairtarget/git-who/internal/cache"
 	"github.com/sinclairtarget/git-who/internal/git"
 )
 
@@ -50,41 +51,10 @@ func (b *GobBackend) compressedPath() string {
 	return b.Path + ".gz"
 }
 
-func (b *GobBackend) Open() (err error) {
+func (b *GobBackend) Open() error {
 	b.wasOpened = true
 
-	// Uncompress gzipped file to regular location if it exists
-	f, err := os.Open(b.compressedPath())
-	if errors.Is(err, fs.ErrNotExist) {
-		return nil
-	} else if err != nil {
-		return err
-	}
-	defer f.Close()
-
-	fout, err := os.OpenFile(b.Path, os.O_WRONLY|os.O_CREATE, 0644)
-	if err != nil {
-		return err
-	}
-	defer fout.Close()
-
-	zr, err := gzip.NewReader(f)
-	if err != nil {
-		return err
-	}
-
-	w := bufio.NewWriter(fout)
-	_, err = io.Copy(w, zr)
-	if err != nil {
-		return err
-	}
-
-	err = zr.Close()
-	if err != nil {
-		return err
-	}
-
-	err = w.Flush()
+	err := uncompress(b.compressedPath(), b.Path)
 	if err != nil {
 		return err
 	}
@@ -92,42 +62,16 @@ func (b *GobBackend) Open() (err error) {
 	return nil
 }
 
-func (b *GobBackend) Close() (err error) {
+func (b *GobBackend) Close() error {
 	if b.isDirty {
-		// Compress file and save to gzipped location
-		f, err := os.Open(b.Path)
-		if errors.Is(err, fs.ErrNotExist) {
-			return nil
-		} else if err != nil {
-			return err
-		}
-		defer f.Close()
-
-		fout, err := os.OpenFile(b.compressedPath(), os.O_WRONLY|os.O_CREATE, 0644)
-		if err != nil {
-			return err
-		}
-		defer fout.Close()
-
-		r := bufio.NewReader(f)
-		zw, err := gzip.NewWriterLevel(fout, gzip.BestSpeed)
-		if err != nil {
-			return err
-		}
-
-		_, err = io.Copy(zw, r)
-		if err != nil {
-			return err
-		}
-
-		err = zw.Close()
+		err := compress(b.Path, b.compressedPath())
 		if err != nil {
 			return err
 		}
 	}
 
 	// Remove uncompressed file
-	err = os.RemoveAll(b.Path)
+	err := os.RemoveAll(b.Path)
 	if err != nil {
 		return err
 	}
@@ -154,23 +98,29 @@ func (b *GobBackend) Close() (err error) {
 	return nil
 }
 
-func (b *GobBackend) Get(revs []string) (_ cache.Result, err error) {
+func (b *GobBackend) Get(revs []string) (iter.Seq[git.Commit], func() error) {
 	if !b.wasOpened {
 		panic("cache not yet open. Did you forget to call Open()?")
 	}
 
-	result := cache.EmptyResult()
-
 	lookingFor := map[string]bool{}
 	for _, rev := range revs {
 		lookingFor[rev] = true
 	}
 
+	empty := slices.Values([]git.Commit{})
+	var iterErr error
+	finish := func() error {
+		return iterErr
+	}
+
 	f, err := os.Open(b.Path)
 	if errors.Is(err, fs.ErrNotExist) {
-		return result, nil
+		// If file doesn't exist, don't treat as an error
+		return empty, finish
 	} else if err != nil {
-		return result, err
+		iterErr = err
+		return empty, finish
 	}
 
 	// In theory we shouldn't get any duplicates into the cache if we're
@@ -178,19 +128,17 @@ func (b *GobBackend) Get(revs []string) (_ cache.Result, err error) {
 	// and throwing an error if we see one.
 	seen := map[string]bool{}
 
-	it := func(yield func(git.Commit, error) bool) {
+	seq := func(yield func(git.Commit) bool) {
 		defer f.Close() // Don't care about error closing when reading
 
 		for {
-			var commit git.Commit
-
 			// -- Find length of next gob in bytes --
 			prefix := make([]byte, 4)
-			_, err = f.Read(prefix)
+			_, err := f.Read(prefix)
 			if err == io.EOF {
 				return
 			} else if err != nil {
-				yield(commit, err)
+				iterErr = err
 				return
 			}
 
@@ -201,7 +149,7 @@ func (b *GobBackend) Get(revs []string) (_ cache.Result, err error) {
 				&size,
 			)
 			if err != nil {
-				yield(commit, err)
+				iterErr = err
 				return
 			}
 
@@ -214,7 +162,7 @@ func (b *GobBackend) Get(revs []string) (_ cache.Result, err error) {
 			dec := gob.NewDecoder(bytes.NewReader(data))
 			err = dec.Decode(&commits)
 			if err != nil {
-				yield(commit, err)
+				iterErr = err
 				return
 			}
 
@@ -223,15 +171,15 @@ func (b *GobBackend) Get(revs []string) (_ cache.Result, err error) {
 				hit, _ := lookingFor[c.Hash]
 				if hit {
 					if isDup, _ := seen[c.Hash]; isDup {
-						yield(c, fmt.Errorf(
+						iterErr = fmt.Errorf(
 							"duplicate commit in cache: %s",
 							c.Hash,
-						))
+						)
 						return
 					}
 
 					seen[c.Hash] = true
-					if !yield(c, nil) {
+					if !yield(c) {
 						return
 					}
 				}
@@ -239,7 +187,7 @@ func (b *GobBackend) Get(revs []string) (_ cache.Result, err error) {
 		}
 	}
 
-	return cache.Result{Commits: it}, nil
+	return seq, finish
 }
 
 func (b *GobBackend) Add(commits []git.Commit) (err error) {
@@ -312,12 +260,81 @@ func GobCacheDir(prefix string, gitRootPath string) string {
 	return repoDir
 }
 
-func GobCacheFilename(repoFiles git.RepoConfigFiles) (string, error) {
-	stateHash, err := cache.RepoStateHash(repoFiles)
+func GobCacheFilename(stateHash string) string {
+	return fmt.Sprintf("%s.gobs", stateHash)
+}
+
+// Uncompress gzipped file to regular location if it exists
+func uncompress(sourcePath string, targetPath string) error {
+	f, err := os.Open(sourcePath)
+	if errors.Is(err, fs.ErrNotExist) {
+		return nil
+	} else if err != nil {
+		return err
+	}
+	defer f.Close()
+
+	fout, err := os.OpenFile(targetPath, os.O_WRONLY|os.O_CREATE, 0644)
 	if err != nil {
-		return "", err
+		return err
+	}
+	defer fout.Close()
+
+	zr, err := gzip.NewReader(f)
+	if err != nil {
+		return err
 	}
 
-	filename := fmt.Sprintf("%s.gobs", stateHash)
-	return filename, nil
+	w := bufio.NewWriter(fout)
+	_, err = io.Copy(w, zr)
+	if err != nil {
+		return err
+	}
+
+	err = zr.Close()
+	if err != nil {
+		return err
+	}
+
+	err = w.Flush()
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// Compress file and save to gzipped location
+func compress(sourcePath string, targetPath string) error {
+	f, err := os.Open(sourcePath)
+	if errors.Is(err, fs.ErrNotExist) {
+		return nil
+	} else if err != nil {
+		return err
+	}
+	defer f.Close()
+
+	fout, err := os.OpenFile(targetPath, os.O_WRONLY|os.O_CREATE, 0644)
+	if err != nil {
+		return err
+	}
+	defer fout.Close()
+
+	r := bufio.NewReader(f)
+	zw, err := gzip.NewWriterLevel(fout, gzip.BestSpeed)
+	if err != nil {
+		return err
+	}
+
+	_, err = io.Copy(zw, r)
+	if err != nil {
+		return err
+	}
+
+	err = zw.Close()
+	if err != nil {
+		return err
+	}
+
+	return nil
 }
