@@ -9,11 +9,11 @@ import (
 	"os"
 	"os/user"
 	"path/filepath"
-	"slices"
 	"time"
 
+	"github.com/sinclairtarget/git-who/internal/cache/backends"
 	"github.com/sinclairtarget/git-who/internal/git"
-	"github.com/sinclairtarget/git-who/internal/utils/iterutils"
+	"github.com/sinclairtarget/git-who/internal/git/config"
 )
 
 func IsCachingEnabled() bool {
@@ -24,23 +24,11 @@ func IsCachingEnabled() bool {
 	return true
 }
 
-type Result struct {
-	Commits iter.Seq2[git.Commit, error] // The sequence of commits
-}
-
-// If we use the zero-value for Result, the iterator will be nil. We instead
-// want an interator over a zero-length sequence.
-func EmptyResult() Result {
-	return Result{
-		Commits: iterutils.WithoutErrors(slices.Values([]git.Commit{})),
-	}
-}
-
 type Backend interface {
 	Name() string
 	Open() error
 	Close() error
-	Get(revs []string) (Result, error)
+	Get(revs []string) (iter.Seq[git.Commit], func() error)
 	Add(commits []git.Commit) error
 	Clear() error
 }
@@ -107,19 +95,10 @@ func (c *Cache) Close() (err error) {
 	return nil
 }
 
-func (c *Cache) Get(revs []string) (_ Result, err error) {
-	defer func() {
-		if err != nil {
-			err = fmt.Errorf("failed to retrieve from cache: %w", err)
-		}
-	}()
-
+func (c *Cache) Get(revs []string) (iter.Seq[git.Commit], func() error) {
 	start := time.Now()
 
-	result, err := c.backend.Get(revs)
-	if err != nil {
-		return result, err
-	}
+	commits, finish := c.backend.Get(revs)
 
 	elapsed := time.Now().Sub(start)
 	logger().Debug(
@@ -128,12 +107,14 @@ func (c *Cache) Get(revs []string) (_ Result, err error) {
 		elapsed.Milliseconds(),
 	)
 
-	// Make sure iterator is not nil
-	if result.Commits == nil {
-		panic("Cache backend returned nil commits iterator; this isn't kosher!")
-	}
+	return commits, func() error {
+		err := finish()
+		if err != nil {
+			err = fmt.Errorf("failed to retrieve from cache: %w", err)
+		}
 
-	return result, nil
+		return err
+	}
 }
 
 func (c *Cache) Add(commits []git.Commit) error {
@@ -168,7 +149,7 @@ func (c *Cache) Clear() error {
 // backend.
 //
 // Tries to store it under the XDG_CACHE_HOME dir.
-func CacheStorageDir(name string) (_ string, err error) {
+func cacheStorageDir(name string) (_ string, err error) {
 	defer func() {
 		if err != nil {
 			err = fmt.Errorf("failed to determine cache storage path: %w", err)
@@ -195,12 +176,51 @@ func CacheStorageDir(name string) (_ string, err error) {
 }
 
 // Hash of all the state in the repo that affects the validity of our cache
-func RepoStateHash(rf git.RepoConfigFiles) (string, error) {
+func repoStateHash(sf config.SupplementalFiles) (string, error) {
 	h := fnv.New32()
-	err := rf.MailmapHash(h)
+	err := sf.MailmapHash(h)
 	if err != nil {
 		return "", err
 	}
 
 	return hex.EncodeToString(h.Sum(nil)), nil
 }
+
+func warnFail(cb Backend, err error) Cache {
+	logger().Warn(
+		fmt.Sprintf("failed to initialize cache: %v", err),
+	)
+	logger().Warn("disabling caching")
+	return NewCache(cb)
+}
+
+func GetCache(gitRootPath string, configFiles config.SupplementalFiles) Cache {
+	var fallback Backend = backends.NoopBackend{}
+
+	if !IsCachingEnabled() {
+		return NewCache(fallback)
+	}
+
+	cacheStorageDir, err := cacheStorageDir(
+		backends.GobBackendName,
+	)
+	if err != nil {
+		return warnFail(fallback, err)
+	}
+
+	dirname := backends.GobCacheDir(cacheStorageDir, gitRootPath)
+	err = os.MkdirAll(dirname, 0o700)
+	if err != nil {
+		return warnFail(fallback, err)
+	}
+
+	stateHash, err := repoStateHash(configFiles)
+	if err != nil {
+		return warnFail(fallback, err)
+	}
+
+	filename := backends.GobCacheFilename(stateHash)
+	p := filepath.Join(dirname, filename)
+	logger().Debug("cache initialized", "path", p)
+	return NewCache(&backends.GobBackend{Path: p, Dir: dirname})
+}
