@@ -30,6 +30,8 @@ import (
 	"github.com/sinclairtarget/git-who/internal/cache"
 	"github.com/sinclairtarget/git-who/internal/format"
 	"github.com/sinclairtarget/git-who/internal/git"
+	"github.com/sinclairtarget/git-who/internal/git/cmd"
+	"github.com/sinclairtarget/git-who/internal/git/config"
 	"github.com/sinclairtarget/git-who/internal/pretty"
 	"github.com/sinclairtarget/git-who/internal/tally"
 )
@@ -44,7 +46,7 @@ func init() {
 }
 
 type tallyFunc[T any] func(
-	commits iter.Seq2[git.Commit, error],
+	commits iter.Seq[git.Commit],
 	opts tally.TallyOpts,
 ) (T, error)
 
@@ -56,7 +58,7 @@ type combinable[T any] interface {
 type whoperation[T combinable[T]] struct {
 	revspec    []string
 	pathspecs  []string
-	filters    git.LogFilters
+	filters    cmd.LogFilters
 	useMailmap bool
 	ignoreRevs []string
 	tally      tallyFunc[T]
@@ -88,6 +90,8 @@ func setDiff(a []string, b []string) []string {
 	return ret
 }
 
+// Tallies the commits in the cache, returning a slice of remaining revs to
+// tally.
 func accumulateCached[T combinable[T]](
 	whop whoperation[T],
 	c cache.Cache,
@@ -95,19 +99,23 @@ func accumulateCached[T combinable[T]](
 ) (T, []string, error) {
 	var none T
 
-	result, err := c.Get(revs)
+	commits, finish := c.Get(revs)
+	commits, err := git.LimitDiffsByPathspec(commits, whop.pathspecs)
 	if err != nil {
 		return none, revs, err
 	}
 
-	commits := git.LimitDiffsByPathspec(result.Commits, whop.pathspecs)
-
 	foundRevs := []string{}
 	accumulator, err := whop.tally(revTee(commits, &foundRevs), whop.opts)
 	if err != nil {
 		return none, revs, err
 	}
 
+	err = finish()
+	if err != nil {
+		return none, revs, err
+	}
+
 	logger().Debug("commits found in cache", "num", len(foundRevs))
 	return accumulator, setDiff(revs, foundRevs), nil
 }
@@ -292,6 +300,14 @@ loop:
 					err,
 				)
 			}
+		case err, ok := <-cacheErr:
+			if ok && err != nil {
+				logger().Debug("cache error in concurrent tally; cancelling")
+				return accumulator, fmt.Errorf(
+					"concurrent tally failed: %w",
+					err,
+				)
+			}
 		}
 	}
 
@@ -300,6 +316,8 @@ loop:
 	}
 
 	// Check if there was a caching error (and wait for cacher to exit)
+	// We have to do this here in addition to above in case there is a caching
+	// error after the last chunk of work is handled.
 	select {
 	case <-ctx.Done():
 		return accumulator, errors.New("concurrent tally cancelled")
@@ -316,13 +334,13 @@ func TallyCommits(
 	ctx context.Context,
 	revspec []string,
 	pathspecs []string,
-	filters git.LogFilters,
-	repoFiles git.RepoConfigFiles,
+	filters cmd.LogFilters,
+	configFiles config.SupplementalFiles,
 	opts tally.TallyOpts,
 	cache cache.Cache,
 	allowProgressBar bool,
 ) (_ map[string]tally.Tally, err error) {
-	ignoreRevs, err := repoFiles.IgnoreRevs()
+	ignoreRevs, err := configFiles.IgnoreRevs()
 	if err != nil {
 		return nil, err
 	}
@@ -331,7 +349,7 @@ func TallyCommits(
 		revspec:    revspec,
 		pathspecs:  pathspecs,
 		filters:    filters,
-		useMailmap: repoFiles.HasMailmap(),
+		useMailmap: configFiles.HasMailmap(),
 		ignoreRevs: ignoreRevs,
 		tally:      tally.TallyCommitsByPath,
 		opts:       opts,
@@ -354,15 +372,15 @@ func TallyCommitsTree(
 	ctx context.Context,
 	revspec []string,
 	pathspecs []string,
-	filters git.LogFilters,
-	repoFiles git.RepoConfigFiles,
+	filters cmd.LogFilters,
+	configFiles config.SupplementalFiles,
 	opts tally.TallyOpts,
 	worktreePaths map[string]bool,
 	gitRootPath string,
 	cache cache.Cache,
 	allowProgressBar bool,
 ) (*tally.TreeNode, error) {
-	ignoreRevs, err := repoFiles.IgnoreRevs()
+	ignoreRevs, err := configFiles.IgnoreRevs()
 	if err != nil {
 		return nil, err
 	}
@@ -371,7 +389,7 @@ func TallyCommitsTree(
 		revspec:    revspec,
 		pathspecs:  pathspecs,
 		filters:    filters,
-		useMailmap: repoFiles.HasMailmap(),
+		useMailmap: configFiles.HasMailmap(),
 		ignoreRevs: ignoreRevs,
 		tally:      tally.TallyCommitsByPath,
 		opts:       opts,
@@ -398,20 +416,20 @@ func TallyCommitsTimeline(
 	ctx context.Context,
 	revspec []string,
 	pathspecs []string,
-	filters git.LogFilters,
-	repoFiles git.RepoConfigFiles,
+	filters cmd.LogFilters,
+	configFiles config.SupplementalFiles,
 	opts tally.TallyOpts,
 	end time.Time,
 	cache cache.Cache,
 	allowProgressBar bool,
 ) ([]tally.TimeBucket, error) {
-	ignoreRevs, err := repoFiles.IgnoreRevs()
+	ignoreRevs, err := configFiles.IgnoreRevs()
 	if err != nil {
 		return nil, err
 	}
 
 	f := func(
-		commits iter.Seq2[git.Commit, error],
+		commits iter.Seq[git.Commit],
 		opts tally.TallyOpts,
 	) (tally.TimeSeries, error) {
 		return tally.TallyCommitsByDate(commits, opts)
@@ -421,7 +439,7 @@ func TallyCommitsTimeline(
 		revspec:    revspec,
 		pathspecs:  pathspecs,
 		filters:    filters,
-		useMailmap: repoFiles.HasMailmap(),
+		useMailmap: configFiles.HasMailmap(),
 		ignoreRevs: ignoreRevs,
 		tally:      f,
 		opts:       opts,
