@@ -7,6 +7,7 @@ import (
 
 	"github.com/sinclairtarget/git-who/internal/cache"
 	"github.com/sinclairtarget/git-who/internal/git"
+	"github.com/sinclairtarget/git-who/internal/git/cmd"
 )
 
 type worker struct {
@@ -187,7 +188,7 @@ loop:
 			// commit. Otherwise when we cache the commits we would be caching
 			// only a part of the commit
 			nopaths := []string{}
-			subprocess, err := git.RunStdinLog(
+			subprocess, err := cmd.RunStdinLog(
 				ctx,
 				nopaths,
 				true,
@@ -211,15 +212,27 @@ loop:
 			}
 
 			// Read parsed commits and enqueue for caching
-			lines := subprocess.StdoutLogLines()
-			commits := git.ParseCommits(lines)
-			commits = cacheTee(commits, toCache)
+			result, err := func() (_ T, err error) {
+				var result T
 
-			// Now that we're tallying, we DO care to only look at the file
-			// diffs under the given paths
-			commits = git.LimitDiffsByPathspec(commits, whop.pathspecs)
+				lines, finish := subprocess.StdoutNullDelimitedLines()
+				defer func() { err = errors.Join(err, finish()) }()
+
+				commits, finish := git.ParseCommits(lines)
+				defer func() { err = errors.Join(err, finish()) }()
+
+				commits = cacheTee(commits, toCache)
+
+				// Now that we're tallying, we DO care to only look at the file
+				// diffs under the given paths
+				commits, err = git.LimitDiffsByPathspec(commits, whop.pathspecs)
+				if err != nil {
+					return result, err
+				}
+
+				return whop.tally(commits, whop.opts)
+			}()
 
-			result, err := whop.tally(commits, whop.opts)
 			if err != nil {
 				return err
 			}
