@@ -8,11 +8,14 @@ package git
 
 import (
 	"context"
+	"errors"
 	"fmt"
-	"io"
 	"iter"
-	"strings"
+	"slices"
 	"time"
+
+	"github.com/sinclairtarget/git-who/internal/git/cmd"
+	"github.com/sinclairtarget/git-who/internal/git/config"
 )
 
 type Commit struct {
@@ -62,53 +65,60 @@ func (d FileDiff) String() string {
 	)
 }
 
-// Returns an iterator over commits identified by the given revisions and paths.
-//
-// Also returns a closer() function for cleanup and an error when encountered.
+// Returns a single-use iterator over commits identified by the given revisions
+// and paths.
 func CommitsWithOpts(
 	ctx context.Context,
 	revs []string,
 	pathspecs []string,
-	filters LogFilters,
+	filters cmd.LogFilters,
 	populateDiffs bool,
-	repoFiles RepoConfigFiles,
+	configFiles config.SupplementalFiles,
 ) (
-	iter.Seq2[Commit, error],
+	iter.Seq[Commit],
 	func() error,
-	error,
 ) {
-	ignoreRevs, err := repoFiles.IgnoreRevs()
+	empty := slices.Values([]Commit{})
+
+	ignoreRevs, err := configFiles.IgnoreRevs()
 	if err != nil {
-		return nil, nil, err
+		return empty, func() error { return err }
 	}
 
-	subprocess, err := RunLog(
+	subprocess, err := cmd.RunLog(
 		ctx,
 		revs,
 		pathspecs,
 		filters,
 		populateDiffs,
-		repoFiles.HasMailmap(),
+		configFiles.HasMailmap(),
 	)
 	if err != nil {
-		return nil, nil, err
+		return empty, func() error { return err }
 	}
 
-	lines := subprocess.StdoutLogLines()
-	commits := ParseCommits(lines)
+	lines, finishLines := subprocess.StdoutNullDelimitedLines()
+	commits, finishCommits := ParseCommits(lines)
 	commits = SkipIgnored(commits, ignoreRevs)
 
-	closer := func() error {
+	finish := func() error {
+		iterErr := finishCommits()
+		iterErr = errors.Join(iterErr, finishLines())
+		if iterErr != nil {
+			return fmt.Errorf("error iterating commits: %v", iterErr)
+		}
+
 		return subprocess.Wait()
 	}
-	return commits, closer, nil
+
+	return commits, finish
 }
 
 func RevList(
 	ctx context.Context,
 	revranges []string,
 	pathspecs []string,
-	filters LogFilters,
+	filters cmd.LogFilters,
 ) (_ []string, err error) {
 	defer func() {
 		if err != nil {
@@ -118,20 +128,21 @@ func RevList(
 
 	revs := []string{}
 
-	subprocess, err := RunRevList(ctx, revranges, pathspecs, filters)
+	subprocess, err := cmd.RunRevList(ctx, revranges, pathspecs, filters)
 	if err != nil {
 		return revs, err
 	}
 
-	lines := subprocess.StdoutLines()
-	for line, err := range lines {
-		if err != nil {
-			return revs, err
-		}
-
+	lines, finish := subprocess.StdoutLines()
+	for line := range lines {
 		revs = append(revs, line)
 	}
 
+	err = finish()
+	if err != nil {
+		return revs, err
+	}
+
 	err = subprocess.Wait()
 	if err != nil {
 		return revs, err
@@ -143,23 +154,19 @@ func RevList(
 func GetRoot() (_ string, err error) {
 	defer func() {
 		if err != nil {
-			err = fmt.Errorf(
-				"failed to run git rev-parse --show-toplevel: %w",
-				err,
-			)
+			err = fmt.Errorf("failed to get Git root directory: %w", err)
 		}
 	}()
 
 	ctx, cancel := context.WithCancel(context.Background())
 	defer cancel()
 
-	args := []string{"rev-parse", "--show-toplevel"}
-	subprocess, err := run(ctx, args, false)
+	subprocess, err := cmd.RunRevParseTopLevel(ctx)
 	if err != nil {
 		return "", err
 	}
 
-	b, err := io.ReadAll(subprocess.stdout)
+	root, err := subprocess.StdoutText()
 	if err != nil {
 		return "", err
 	}
@@ -169,7 +176,6 @@ func GetRoot() (_ string, err error) {
 		return "", err
 	}
 
-	root := strings.TrimSpace(string(b))
 	return root, nil
 }
 
@@ -186,17 +192,19 @@ func WorkingTreeFiles(pathspecs []string) (_ map[string]bool, err error) {
 
 	wtreeset := map[string]bool{}
 
-	subprocess, err := RunLsFiles(ctx, pathspecs)
+	subprocess, err := cmd.RunLsFiles(ctx, pathspecs)
 	if err != nil {
 		return wtreeset, err
 	}
 
-	lines := subprocess.StdoutLines()
-	for line, err := range lines {
-		if err != nil {
-			return wtreeset, err
-		}
-		wtreeset[strings.TrimSpace(line)] = true
+	lines, finish := subprocess.StdoutNullDelimitedLines()
+	for line := range lines {
+		wtreeset[line] = true
+	}
+
+	err = finish()
+	if err != nil {
+		return wtreeset, err
 	}
 
 	err = subprocess.Wait()
@@ -210,36 +218,30 @@ func WorkingTreeFiles(pathspecs []string) (_ map[string]bool, err error) {
 // Returns all commits in the input iterator, but for each commit, strips out
 // any file diff not modifying one of the given pathspecs
 func LimitDiffsByPathspec(
-	commits iter.Seq2[Commit, error],
+	commits iter.Seq[Commit],
 	pathspecs []string,
-) iter.Seq2[Commit, error] {
+) (iter.Seq[Commit], error) {
 	if len(pathspecs) == 0 {
-		return commits
+		return commits, nil
 	}
 
-	return func(yield func(Commit, error) bool) {
-		// Check all pathspecs are supported
-		for _, p := range pathspecs {
-			if !IsSupportedPathspec(p) {
-				yield(
-					Commit{},
-					fmt.Errorf("unsupported magic in pathspec: \"%s\"", p),
-				)
-				return
-			}
+	// Check all pathspecs are supported
+	for _, p := range pathspecs {
+		if !IsSupportedPathspec(p) {
+			err := fmt.Errorf("unsupported magic in pathspec: \"%s\"", p)
+			return commits, err
 		}
+	}
 
+	return func(yield func(Commit) bool) {
 		includes, excludes := SplitPathspecs(pathspecs)
 
-		for commit, err := range commits {
-			if err != nil {
-				yield(commit, err)
-				return
-			}
-
+		for commit := range commits {
 			filtered := []FileDiff{}
 			for _, diff := range commit.FileDiffs {
-				shouldInclude := false
+				// If we don't have any explicit includes, then everything is
+				// included.
+				shouldInclude := len(includes) == 0
 				for _, p := range includes {
 					if PathspecMatch(p, diff.Path) {
 						shouldInclude = true
@@ -261,33 +263,30 @@ func LimitDiffsByPathspec(
 			}
 
 			commit.FileDiffs = filtered
-			yield(commit, nil)
+			if !yield(commit) {
+				return
+			}
 		}
-	}
+	}, nil
 }
 
 // Returns an iterator over commits that skips any revs in the given list.
 func SkipIgnored(
-	commits iter.Seq2[Commit, error],
+	commits iter.Seq[Commit],
 	ignoreRevs []string,
-) iter.Seq2[Commit, error] {
+) iter.Seq[Commit] {
 	ignoreSet := map[string]bool{}
 	for _, rev := range ignoreRevs {
 		ignoreSet[rev] = true
 	}
 
-	return func(yield func(Commit, error) bool) {
-		for commit, err := range commits {
-			if err != nil {
-				yield(commit, err)
-				return
-			}
-
+	return func(yield func(Commit) bool) {
+		for commit := range commits {
 			if shouldIgnore := ignoreSet[commit.Hash]; shouldIgnore {
 				continue // skip this commit
 			}
 
-			if !yield(commit, nil) {
+			if !yield(commit) {
 				break
 			}
 		}
