@@ -7,9 +7,9 @@ import (
 	"log/slog"
 	"os"
 	"strings"
-	"time"
 
 	"github.com/sinclairtarget/git-who/internal/git"
+	"github.com/sinclairtarget/git-who/internal/subcommands"
 	"github.com/sinclairtarget/git-who/internal/tally"
 	"github.com/sinclairtarget/git-who/internal/utils/flagutils"
 )
@@ -17,8 +17,6 @@ import (
 var Commit = "unknown"
 var Version = "unknown"
 
-var progStart time.Time
-
 type command struct {
 	flagSet     *flag.FlagSet
 	run         func(args []string) error
@@ -61,6 +59,9 @@ func main() {
 			fmt.Printf("  %s\n", name)
 			fmt.Printf("\t%s\n", cmd.description)
 		}
+
+		fmt.Println()
+		fmt.Println("Run git-who <subcommand> -h for subcommand help")
 	}
 
 	// Look for the index of the first arg not intended as a top-level flag.
@@ -109,7 +110,6 @@ loop:
 	subargs := cmd.flagSet.Args()
 	subargs = unescapeTerminator(subargs)
 
-	progStart = time.Now()
 	if err := cmd.run(subargs); err != nil {
 		fmt.Fprintf(os.Stderr, "%s\n", err)
 		os.Exit(1)
@@ -182,7 +182,7 @@ Usage: git-who table [options...] [revisions...] [[--] paths...]
 				return err
 			}
 
-			return table(
+			return subcommands.Table(
 				revs,
 				pathspecs,
 				mode,
@@ -203,7 +203,7 @@ func treeCmd() command {
 	flagSet := flag.NewFlagSet("git-who tree", flag.ExitOnError)
 
 	showEmail := flagSet.Bool("e", false, "Show email address of each author")
-	showHidden := flagSet.Bool("a", false, "Show files not in working tree")
+	showHidden := flagSet.Bool("a", false, "Show files not in working tree (also annotates all files)")
 	countMerges := flagSet.Bool("merges", false, "Count merge commits toward commit total")
 	useLines := flagSet.Bool("l", false, "Rank authors by lines added/changed")
 	useFiles := flagSet.Bool("f", false, "Rank authors by files touched")
@@ -262,7 +262,7 @@ Usage: git-who tree [options...] [revisions...] [[--] paths...]
 				mode = tally.FirstModifiedMode
 			}
 
-			return tree(
+			return subcommands.Tree(
 				revs,
 				pathspecs,
 				mode,
@@ -325,7 +325,7 @@ Usage: git-who hist [options...] [revisions...] [[--] paths...]
 				mode = tally.FilesMode
 			}
 
-			return hist(
+			return subcommands.Hist(
 				revs,
 				pathspecs,
 				mode,
@@ -360,7 +360,7 @@ func dumpCmd() command {
 				return err
 			}
 
-			return dump(
+			return subcommands.Dump(
 				revs,
 				pathspecs,
 				*short,
@@ -393,7 +393,7 @@ func parseCmd() command {
 				return err
 			}
 
-			return parse(
+			return subcommands.Parse(
 				revs,
 				pathspecs,
 				*short,
